from PySide6.QtWidgets import (
    QApplication, QMainWindow, QTextEdit, QVBoxLayout, QWidget
)
from PySide6.QtCore import Qt, QMimeData
from fontTools.ttLib import TTFont
import sys


def extract_font_characters(font_path):
    font = TTFont(font_path)
    cmap = font['cmap']
    chars = set()

    for table in cmap.tables:
        if table.isUnicode():
            chars.update(chr(cp) for cp in table.cmap.keys())

    return ''.join(sorted(chars))


class DropTextEdit(QTextEdit):
    def __init__(self):
        super().__init__()
        self.setAcceptDrops(True)
        self.setReadOnly(True)
        self.setPlaceholderText("ここにフォントファイル（.ttf, .otf）をドロップ")

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()

    def dragMoveEvent(self, event):
        event.acceptProposedAction()

    def dropEvent(self, event):
        for url in event.mimeData().urls():
            path = url.toLocalFile()
            if path.lower().endswith(('.ttf', '.otf')):
                try:
                    chars = extract_font_characters(path)
                    self.setPlainText(chars)
                except Exception as e:
                    self.setPlainText(f"エラー: {e}")
                break


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("フォント文字一覧ビューア")
        self.resize(600, 400)
        self.setAcceptDrops(True)  # 念のためウィンドウ自体も対応

        self.drop_area = DropTextEdit()

        layout = QVBoxLayout()
        layout.addWidget(self.drop_area)

        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec())
